@@ -77,6 +77,7 @@ typedef struct _GeeSetIface GeeSetIface;
 typedef struct _GeeReadOnlySet GeeReadOnlySet;
 typedef struct _GeeReadOnlySetClass GeeReadOnlySetClass;
 typedef struct _GeeReadOnlySetPrivate GeeReadOnlySetPrivate;
+#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
 
 #define GEE_READ_ONLY_SET_TYPE_ITERATOR (gee_read_only_set_iterator_get_type ())
 #define GEE_READ_ONLY_SET_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIterator))
@@ -89,9 +90,6 @@ typedef struct _GeeReadOnlySetIterator GeeReadOnlySetIterator;
 typedef struct _GeeReadOnlySetIteratorClass GeeReadOnlySetIteratorClass;
 typedef struct _GeeReadOnlySetIteratorPrivate GeeReadOnlySetIteratorPrivate;
 
-/**
- * Base class for all collections.
- */
 struct _GeeCollectionObject {
 	GTypeInstance parent_instance;
 	volatile int ref_count;
@@ -103,30 +101,18 @@ struct _GeeCollectionObjectClass {
 	void (*finalize) (GeeCollectionObject *self);
 };
 
-/**
- * Implemented by classes that support a simple iteration over instances of the
- * collection.
- */
 struct _GeeIteratorIface {
 	GTypeInterface parent_iface;
 	gboolean (*next) (GeeIterator* self);
 	gpointer (*get) (GeeIterator* self);
 };
 
-/**
- * Implemented by classes that support a simple iteration over instances of the
- * collection.
- */
 struct _GeeIterableIface {
 	GTypeInterface parent_iface;
 	GType (*get_element_type) (GeeIterable* self);
 	GeeIterator* (*iterator) (GeeIterable* self);
 };
 
-/**
- * Serves as the base interface for implementing collection classes. Defines
- * size, iteration, and modification methods.
- */
 struct _GeeCollectionIface {
 	GTypeInterface parent_iface;
 	gboolean (*contains) (GeeCollection* self, gconstpointer item);
@@ -136,16 +122,10 @@ struct _GeeCollectionIface {
 	gint (*get_size) (GeeCollection* self);
 };
 
-/**
- * A set is a collection without duplicates.
- */
 struct _GeeSetIface {
 	GTypeInterface parent_iface;
 };
 
-/**
- * Represents a read-only collection of items without duplicates.
- */
 struct _GeeReadOnlySet {
 	GeeCollectionObject parent_instance;
 	GeeReadOnlySetPrivate * priv;
@@ -178,6 +158,12 @@ struct _GeeReadOnlySetIteratorPrivate {
 };
 
 
+static gpointer gee_read_only_set_iterator_parent_class = NULL;
+static GeeIteratorIface* gee_read_only_set_iterator_gee_iterator_parent_iface = NULL;
+static gpointer gee_read_only_set_parent_class = NULL;
+static GeeIterableIface* gee_read_only_set_gee_iterable_parent_iface = NULL;
+static GeeCollectionIface* gee_read_only_set_gee_collection_parent_iface = NULL;
+static GeeSetIface* gee_read_only_set_gee_set_parent_iface = NULL;
 
 gpointer gee_collection_object_ref (gpointer instance);
 void gee_collection_object_unref (gpointer instance);
@@ -194,10 +180,11 @@ GType gee_read_only_set_get_type (void);
 enum  {
 	GEE_READ_ONLY_SET_DUMMY_PROPERTY
 };
+GeeCollectionObject* gee_collection_object_new (void);
+GeeCollectionObject* gee_collection_object_construct (GType object_type);
 void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value);
 GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
 GeeReadOnlySet* gee_read_only_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
-GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
 static GType gee_read_only_set_real_get_element_type (GeeIterable* base);
 static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
 static GeeReadOnlySetIterator* gee_read_only_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
@@ -216,20 +203,13 @@ enum  {
 };
 static gboolean gee_read_only_set_iterator_real_next (GeeIterator* base);
 static gpointer gee_read_only_set_iterator_real_get (GeeIterator* base);
-static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
-static gpointer gee_read_only_set_iterator_parent_class = NULL;
-static GeeIteratorIface* gee_read_only_set_iterator_gee_iterator_parent_iface = NULL;
-static gpointer gee_read_only_set_parent_class = NULL;
-static GeeIterableIface* gee_read_only_set_gee_iterable_parent_iface = NULL;
-static GeeCollectionIface* gee_read_only_set_gee_collection_parent_iface = NULL;
-static GeeSetIface* gee_read_only_set_gee_set_parent_iface = NULL;
 static void gee_read_only_set_finalize (GeeCollectionObject* obj);
 
 
 
 GeeReadOnlySet* gee_read_only_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set) {
 	GeeReadOnlySet* self;
-	self = (GeeReadOnlySet*) g_type_create_instance (object_type);
+	self = (GeeReadOnlySet*) gee_collection_object_construct (object_type);
 	self->priv->g_type = g_type;
 	self->priv->g_dup_func = g_dup_func;
 	self->priv->g_destroy_func = g_destroy_func;
@@ -245,28 +225,36 @@ GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func,
 
 static GType gee_read_only_set_real_get_element_type (GeeIterable* base) {
 	GeeReadOnlySet * self;
+	GType result;
 	self = (GeeReadOnlySet*) base;
-	return self->priv->g_type;
+	result = self->priv->g_type;
+	return result;
 }
 
 
 static GeeIterator* gee_read_only_set_real_iterator (GeeIterable* base) {
 	GeeReadOnlySet * self;
+	GeeIterator* result;
 	self = (GeeReadOnlySet*) base;
 	if (self->priv->_set == NULL) {
-		return (GeeIterator*) gee_read_only_set_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		result = (GeeIterator*) gee_read_only_set_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		return result;
 	}
-	return gee_iterable_iterator ((GeeIterable*) self->priv->_set);
+	result = gee_iterable_iterator ((GeeIterable*) self->priv->_set);
+	return result;
 }
 
 
 static gboolean gee_read_only_set_real_contains (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlySet * self;
+	gboolean result;
 	self = (GeeReadOnlySet*) base;
 	if (self->priv->_set == NULL) {
-		return FALSE;
+		result = FALSE;
+		return result;
 	}
-	return gee_collection_contains ((GeeCollection*) self->priv->_set, item);
+	result = gee_collection_contains ((GeeCollection*) self->priv->_set, item);
+	return result;
 }
 
 
@@ -292,39 +280,47 @@ static void gee_read_only_set_real_clear (GeeCollection* base) {
 
 
 static gint gee_read_only_set_real_get_size (GeeCollection* base) {
+	gint result;
 	GeeReadOnlySet* self;
 	self = (GeeReadOnlySet*) base;
-	return gee_collection_get_size ((GeeCollection*) self->priv->_set);
+	result = gee_collection_get_size ((GeeCollection*) self->priv->_set);
+	return result;
+}
+
+
+static gpointer _gee_collection_object_ref0 (gpointer self) {
+	return self ? gee_collection_object_ref (self) : NULL;
 }
 
 
 void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value) {
-	GeeSet* _tmp1_;
 	GeeSet* _tmp0_;
 	g_return_if_fail (self != NULL);
-	_tmp1_ = NULL;
-	_tmp0_ = NULL;
-	self->priv->_set = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : gee_collection_object_ref (_tmp0_)), (self->priv->_set == NULL) ? NULL : (self->priv->_set = (gee_collection_object_unref (self->priv->_set), NULL)), _tmp1_);
+	self->priv->_set = (_tmp0_ = _gee_collection_object_ref0 (value), _gee_collection_object_unref0 (self->priv->_set), _tmp0_);
 }
 
 
 static gboolean gee_read_only_set_iterator_real_next (GeeIterator* base) {
 	GeeReadOnlySetIterator * self;
+	gboolean result;
 	self = (GeeReadOnlySetIterator*) base;
-	return FALSE;
+	result = FALSE;
+	return result;
 }
 
 
 static gpointer gee_read_only_set_iterator_real_get (GeeIterator* base) {
 	GeeReadOnlySetIterator * self;
+	gpointer result;
 	self = (GeeReadOnlySetIterator*) base;
-	return NULL;
+	result = NULL;
+	return result;
 }
 
 
 static GeeReadOnlySetIterator* gee_read_only_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func) {
 	GeeReadOnlySetIterator* self;
-	self = (GeeReadOnlySetIterator*) g_type_create_instance (object_type);
+	self = (GeeReadOnlySetIterator*) gee_collection_object_construct (object_type);
 	self->priv->g_type = g_type;
 	self->priv->g_dup_func = g_dup_func;
 	self->priv->g_destroy_func = g_destroy_func;
@@ -404,7 +400,7 @@ static void gee_read_only_set_instance_init (GeeReadOnlySet * self) {
 static void gee_read_only_set_finalize (GeeCollectionObject* obj) {
 	GeeReadOnlySet * self;
 	self = GEE_READ_ONLY_SET (obj);
-	(self->priv->_set == NULL) ? NULL : (self->priv->_set = (gee_collection_object_unref (self->priv->_set), NULL));
+	_gee_collection_object_unref0 (self->priv->_set);
 	GEE_COLLECTION_OBJECT_CLASS (gee_read_only_set_parent_class)->finalize (obj);
 }
 
