@@ -210,6 +210,7 @@ typedef struct _ValaArrayResizeMethodClass ValaArrayResizeMethodClass;
 
 typedef struct _ValaArrayMoveMethod ValaArrayMoveMethod;
 typedef struct _ValaArrayMoveMethodClass ValaArrayMoveMethodClass;
+#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
 
 #define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
 #define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
@@ -302,6 +303,8 @@ typedef struct _ValaIntegerTypeClass ValaIntegerTypeClass;
 
 typedef struct _ValaVoidType ValaVoidType;
 typedef struct _ValaVoidTypeClass ValaVoidTypeClass;
+#define _g_free0(var) (var = (g_free (var), NULL))
+#define _vala_code_context_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_context_unref (var), NULL)))
 
 #define VALA_TYPE_POINTER_TYPE (vala_pointer_type_get_type ())
 #define VALA_POINTER_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_POINTER_TYPE, ValaPointerType))
@@ -323,12 +326,6 @@ typedef struct _ValaPointerTypeClass ValaPointerTypeClass;
 typedef struct _ValaTypeParameter ValaTypeParameter;
 typedef struct _ValaTypeParameterClass ValaTypeParameterClass;
 
-/**
- * Represents a part of the parsed source code.
- *
- * Code nodes get created by the parser and are used throughout the whole
- * compilation process.
- */
 struct _ValaCodeNode {
 	GTypeInstance parent_instance;
 	volatile int ref_count;
@@ -349,10 +346,6 @@ struct _ValaCodeNodeClass {
 	void (*get_used_variables) (ValaCodeNode* self, GeeCollection* collection);
 };
 
-/**
- * A reference to a data type. This is used to specify static types of
- * expressions.
- */
 struct _ValaDataType {
 	ValaCodeNode parent_instance;
 	ValaDataTypePrivate * priv;
@@ -382,9 +375,6 @@ struct _ValaDataTypeClass {
 	gboolean (*is_disposable) (ValaDataType* self);
 };
 
-/**
- * A reference type, i.e. a class, interface, or array type.
- */
 struct _ValaReferenceType {
 	ValaDataType parent_instance;
 	ValaReferenceTypePrivate * priv;
@@ -394,9 +384,6 @@ struct _ValaReferenceTypeClass {
 	ValaDataTypeClass parent_class;
 };
 
-/**
- * An array type.
- */
 struct _ValaArrayType {
 	ValaReferenceType parent_instance;
 	ValaArrayTypePrivate * priv;
@@ -407,6 +394,8 @@ struct _ValaArrayTypeClass {
 };
 
 struct _ValaArrayTypePrivate {
+	gboolean _invalid_syntax;
+	gboolean _inline_allocated;
 	gboolean _fixed_length;
 	gint _length;
 	gint _rank;
@@ -424,6 +413,7 @@ typedef enum  {
 } ValaSymbolAccessibility;
 
 
+static gpointer vala_array_type_parent_class = NULL;
 
 gpointer vala_code_node_ref (gpointer instance);
 void vala_code_node_unref (gpointer instance);
@@ -462,6 +452,7 @@ GType vala_array_move_method_get_type (void);
 enum  {
 	VALA_ARRAY_TYPE_DUMMY_PROPERTY
 };
+ValaReferenceType* vala_reference_type_construct (GType object_type);
 void vala_array_type_set_element_type (ValaArrayType* self, ValaDataType* value);
 void vala_array_type_set_rank (ValaArrayType* self, gint value);
 gpointer vala_source_reference_ref (gpointer instance);
@@ -473,7 +464,6 @@ GType vala_source_reference_get_type (void);
 void vala_code_node_set_source_reference (ValaCodeNode* self, ValaSourceReference* value);
 ValaArrayType* vala_array_type_new (ValaDataType* element_type, gint rank, ValaSourceReference* source_reference);
 ValaArrayType* vala_array_type_construct (GType object_type, ValaDataType* element_type, gint rank, ValaSourceReference* source_reference);
-ValaArrayType* vala_array_type_new (ValaDataType* element_type, gint rank, ValaSourceReference* source_reference);
 static ValaArrayLengthField* vala_array_type_get_length_field (ValaArrayType* self);
 static ValaArrayMoveMethod* vala_array_type_get_move_method (ValaArrayType* self);
 static ValaArrayResizeMethod* vala_array_type_get_resize_method (ValaArrayType* self);
@@ -530,6 +520,8 @@ gboolean vala_data_type_get_nullable (ValaDataType* self);
 void vala_data_type_set_nullable (ValaDataType* self, gboolean value);
 gboolean vala_data_type_get_floating_reference (ValaDataType* self);
 void vala_data_type_set_floating_reference (ValaDataType* self, gboolean value);
+gboolean vala_array_type_get_inline_allocated (ValaArrayType* self);
+void vala_array_type_set_inline_allocated (ValaArrayType* self, gboolean value);
 gboolean vala_array_type_get_fixed_length (ValaArrayType* self);
 void vala_array_type_set_fixed_length (ValaArrayType* self, gboolean value);
 gint vala_array_type_get_length (ValaArrayType* self);
@@ -541,8 +533,10 @@ static char* vala_array_type_real_get_cdeclarator_suffix (ValaDataType* base);
 static gboolean vala_array_type_real_is_array (ValaDataType* base);
 char* vala_data_type_to_qualified_string (ValaDataType* self, ValaScope* scope);
 static char* vala_array_type_real_to_qualified_string (ValaDataType* base, ValaScope* scope);
-GType vala_pointer_type_get_type (void);
+char* vala_data_type_get_type_id (ValaDataType* self);
 ValaTypeSymbol* vala_data_type_get_data_type (ValaDataType* self);
+ValaCodeContext* vala_code_context_get (void);
+GType vala_pointer_type_get_type (void);
 ValaAttribute* vala_code_node_get_attribute (ValaCodeNode* self, const char* name);
 GType vala_typeparameter_get_type (void);
 ValaTypeParameter* vala_data_type_get_type_parameter (ValaDataType* self);
@@ -556,8 +550,14 @@ static void vala_array_type_real_accept_children (ValaCodeNode* base, ValaCodeVi
 static void vala_array_type_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
 GeeList* vala_data_type_get_symbols (ValaDataType* self);
 static GeeList* vala_array_type_real_get_symbols (ValaDataType* base);
+gboolean vala_array_type_get_invalid_syntax (ValaArrayType* self);
+void vala_report_error (ValaSourceReference* source, const char* message);
+void vala_code_node_set_error (ValaCodeNode* self, gboolean value);
+gboolean vala_code_node_check (ValaCodeNode* self, ValaSemanticAnalyzer* analyzer);
+static gboolean vala_array_type_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer);
+static char* vala_array_type_real_get_type_id (ValaDataType* base);
 void vala_code_node_set_parent_node (ValaCodeNode* self, ValaCodeNode* value);
-static gpointer vala_array_type_parent_class = NULL;
+void vala_array_type_set_invalid_syntax (ValaArrayType* self, gboolean value);
 static void vala_array_type_finalize (ValaCodeNode* obj);
 static int _vala_strcmp0 (const char * str1, const char * str2);
 
@@ -566,7 +566,7 @@ static int _vala_strcmp0 (const char * str1, const char * str2);
 ValaArrayType* vala_array_type_construct (GType object_type, ValaDataType* element_type, gint rank, ValaSourceReference* source_reference) {
 	ValaArrayType* self;
 	g_return_val_if_fail (element_type != NULL, NULL);
-	self = (ValaArrayType*) g_type_create_instance (object_type);
+	self = (ValaArrayType*) vala_reference_type_construct (object_type);
 	vala_array_type_set_element_type (self, element_type);
 	vala_array_type_set_rank (self, rank);
 	vala_code_node_set_source_reference ((ValaCodeNode*) self, source_reference);
@@ -581,199 +581,196 @@ ValaArrayType* vala_array_type_new (ValaDataType* element_type, gint rank, ValaS
 
 static ValaSymbol* vala_array_type_real_get_member (ValaDataType* base, const char* member_name) {
 	ValaArrayType * self;
+	ValaSymbol* result;
 	self = (ValaArrayType*) base;
 	g_return_val_if_fail (member_name != NULL, NULL);
 	if (_vala_strcmp0 (member_name, "length") == 0) {
-		return (ValaSymbol*) vala_array_type_get_length_field (self);
+		result = (ValaSymbol*) vala_array_type_get_length_field (self);
+		return result;
 	} else {
 		if (_vala_strcmp0 (member_name, "move") == 0) {
-			return (ValaSymbol*) vala_array_type_get_move_method (self);
+			result = (ValaSymbol*) vala_array_type_get_move_method (self);
+			return result;
 		} else {
 			if (_vala_strcmp0 (member_name, "resize") == 0) {
-				return (ValaSymbol*) vala_array_type_get_resize_method (self);
+				result = (ValaSymbol*) vala_array_type_get_resize_method (self);
+				return result;
 			}
 		}
 	}
-	return NULL;
+	result = NULL;
+	return result;
+}
+
+
+static gpointer _vala_code_node_ref0 (gpointer self) {
+	return self ? vala_code_node_ref (self) : NULL;
 }
 
 
 static ValaArrayLengthField* vala_array_type_get_length_field (ValaArrayType* self) {
-	ValaArrayLengthField* _tmp7_;
+	ValaArrayLengthField* result;
 	g_return_val_if_fail (self != NULL, NULL);
 	if (self->priv->length_field == NULL) {
 		ValaArrayLengthField* _tmp0_;
-		ValaNamespace* _tmp1_;
 		ValaNamespace* root_symbol;
-		_tmp0_ = NULL;
-		self->priv->length_field = (_tmp0_ = vala_array_length_field_new (vala_code_node_get_source_reference ((ValaCodeNode*) self)), (self->priv->length_field == NULL) ? NULL : (self->priv->length_field = (vala_code_node_unref (self->priv->length_field), NULL)), _tmp0_);
+		self->priv->length_field = (_tmp0_ = vala_array_length_field_new (vala_code_node_get_source_reference ((ValaCodeNode*) self)), _vala_code_node_unref0 (self->priv->length_field), _tmp0_);
 		vala_symbol_set_access ((ValaSymbol*) self->priv->length_field, VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
-		_tmp1_ = NULL;
-		root_symbol = (_tmp1_ = vala_code_context_get_root (vala_source_file_get_context (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) self)))), (_tmp1_ == NULL) ? NULL : vala_code_node_ref (_tmp1_));
+		root_symbol = _vala_code_node_ref0 (vala_code_context_get_root (vala_source_file_get_context (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) self)))));
 		if (self->priv->_rank > 1) {
-			ValaStruct* _tmp2_;
-			ValaValueType* _tmp3_;
+			ValaStruct* _tmp1_;
+			ValaValueType* _tmp2_;
 			ValaValueType* integer;
-			ValaArrayType* _tmp4_;
-			/* length is an int[] containing the dimensions of the array, starting at 0*/
-			_tmp2_ = NULL;
-			_tmp3_ = NULL;
-			integer = (_tmp3_ = (ValaValueType*) vala_integer_type_new (_tmp2_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) root_symbol), "int")), NULL, NULL), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (vala_code_node_unref (_tmp2_), NULL)), _tmp3_);
-			_tmp4_ = NULL;
-			vala_field_set_field_type ((ValaField*) self->priv->length_field, (ValaDataType*) (_tmp4_ = vala_array_type_new ((ValaDataType*) integer, 1, vala_code_node_get_source_reference ((ValaCodeNode*) self))));
-			(_tmp4_ == NULL) ? NULL : (_tmp4_ = (vala_code_node_unref (_tmp4_), NULL));
-			(integer == NULL) ? NULL : (integer = (vala_code_node_unref (integer), NULL));
+			ValaArrayType* _tmp3_;
+			integer = (_tmp2_ = (ValaValueType*) vala_integer_type_new (_tmp1_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) root_symbol), "int")), NULL, NULL), _vala_code_node_unref0 (_tmp1_), _tmp2_);
+			vala_field_set_field_type ((ValaField*) self->priv->length_field, (ValaDataType*) (_tmp3_ = vala_array_type_new ((ValaDataType*) integer, 1, vala_code_node_get_source_reference ((ValaCodeNode*) self))));
+			_vala_code_node_unref0 (_tmp3_);
+			_vala_code_node_unref0 (integer);
 		} else {
-			ValaIntegerType* _tmp6_;
-			ValaStruct* _tmp5_;
-			_tmp6_ = NULL;
-			_tmp5_ = NULL;
-			vala_field_set_field_type ((ValaField*) self->priv->length_field, (ValaDataType*) (_tmp6_ = vala_integer_type_new (_tmp5_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) root_symbol), "int")), NULL, NULL)));
-			(_tmp6_ == NULL) ? NULL : (_tmp6_ = (vala_code_node_unref (_tmp6_), NULL));
-			(_tmp5_ == NULL) ? NULL : (_tmp5_ = (vala_code_node_unref (_tmp5_), NULL));
+			ValaIntegerType* _tmp5_;
+			ValaStruct* _tmp4_;
+			vala_field_set_field_type ((ValaField*) self->priv->length_field, (ValaDataType*) (_tmp5_ = vala_integer_type_new (_tmp4_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) root_symbol), "int")), NULL, NULL)));
+			_vala_code_node_unref0 (_tmp5_);
+			_vala_code_node_unref0 (_tmp4_);
 		}
-		(root_symbol == NULL) ? NULL : (root_symbol = (vala_code_node_unref (root_symbol), NULL));
+		_vala_code_node_unref0 (root_symbol);
 	}
-	_tmp7_ = NULL;
-	return (_tmp7_ = self->priv->length_field, (_tmp7_ == NULL) ? NULL : vala_code_node_ref (_tmp7_));
+	result = _vala_code_node_ref0 (self->priv->length_field);
+	return result;
 }
 
 
 static ValaArrayResizeMethod* vala_array_type_get_resize_method (ValaArrayType* self) {
-	ValaArrayResizeMethod* _tmp6_;
+	ValaArrayResizeMethod* result;
 	g_return_val_if_fail (self != NULL, NULL);
 	if (self->priv->resize_method == NULL) {
 		ValaArrayResizeMethod* _tmp0_;
 		ValaVoidType* _tmp1_;
-		ValaNamespace* _tmp2_;
 		ValaNamespace* root_symbol;
-		ValaStruct* _tmp3_;
-		ValaIntegerType* _tmp4_;
+		ValaStruct* _tmp2_;
+		ValaIntegerType* _tmp3_;
 		ValaIntegerType* int_type;
-		ValaFormalParameter* _tmp5_;
-		_tmp0_ = NULL;
-		self->priv->resize_method = (_tmp0_ = vala_array_resize_method_new (vala_code_node_get_source_reference ((ValaCodeNode*) self)), (self->priv->resize_method == NULL) ? NULL : (self->priv->resize_method = (vala_code_node_unref (self->priv->resize_method), NULL)), _tmp0_);
-		_tmp1_ = NULL;
+		ValaFormalParameter* _tmp4_;
+		self->priv->resize_method = (_tmp0_ = vala_array_resize_method_new (vala_code_node_get_source_reference ((ValaCodeNode*) self)), _vala_code_node_unref0 (self->priv->resize_method), _tmp0_);
 		vala_method_set_return_type ((ValaMethod*) self->priv->resize_method, (ValaDataType*) (_tmp1_ = vala_void_type_new (NULL)));
-		(_tmp1_ == NULL) ? NULL : (_tmp1_ = (vala_code_node_unref (_tmp1_), NULL));
+		_vala_code_node_unref0 (_tmp1_);
 		vala_symbol_set_access ((ValaSymbol*) self->priv->resize_method, VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
 		vala_method_set_cname ((ValaMethod*) self->priv->resize_method, "g_renew");
-		_tmp2_ = NULL;
-		root_symbol = (_tmp2_ = vala_code_context_get_root (vala_source_file_get_context (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) self)))), (_tmp2_ == NULL) ? NULL : vala_code_node_ref (_tmp2_));
-		_tmp3_ = NULL;
-		_tmp4_ = NULL;
-		int_type = (_tmp4_ = vala_integer_type_new (_tmp3_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) root_symbol), "int")), NULL, NULL), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (vala_code_node_unref (_tmp3_), NULL)), _tmp4_);
-		_tmp5_ = NULL;
-		vala_method_add_parameter ((ValaMethod*) self->priv->resize_method, _tmp5_ = vala_formal_parameter_new ("length", (ValaDataType*) int_type, NULL));
-		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (vala_code_node_unref (_tmp5_), NULL));
+		root_symbol = _vala_code_node_ref0 (vala_code_context_get_root (vala_source_file_get_context (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) self)))));
+		int_type = (_tmp3_ = vala_integer_type_new (_tmp2_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) root_symbol), "int")), NULL, NULL), _vala_code_node_unref0 (_tmp2_), _tmp3_);
+		vala_method_add_parameter ((ValaMethod*) self->priv->resize_method, _tmp4_ = vala_formal_parameter_new ("length", (ValaDataType*) int_type, NULL));
+		_vala_code_node_unref0 (_tmp4_);
 		vala_method_set_returns_modified_pointer ((ValaMethod*) self->priv->resize_method, TRUE);
-		(root_symbol == NULL) ? NULL : (root_symbol = (vala_code_node_unref (root_symbol), NULL));
-		(int_type == NULL) ? NULL : (int_type = (vala_code_node_unref (int_type), NULL));
+		_vala_code_node_unref0 (root_symbol);
+		_vala_code_node_unref0 (int_type);
 	}
-	_tmp6_ = NULL;
-	return (_tmp6_ = self->priv->resize_method, (_tmp6_ == NULL) ? NULL : vala_code_node_ref (_tmp6_));
+	result = _vala_code_node_ref0 (self->priv->resize_method);
+	return result;
 }
 
 
 static ValaArrayMoveMethod* vala_array_type_get_move_method (ValaArrayType* self) {
-	ValaArrayMoveMethod* _tmp8_;
+	ValaArrayMoveMethod* result;
 	g_return_val_if_fail (self != NULL, NULL);
 	if (self->priv->move_method == NULL) {
 		ValaArrayMoveMethod* _tmp0_;
 		ValaVoidType* _tmp1_;
-		ValaNamespace* _tmp2_;
 		ValaNamespace* root_symbol;
-		ValaStruct* _tmp3_;
-		ValaIntegerType* _tmp4_;
+		ValaStruct* _tmp2_;
+		ValaIntegerType* _tmp3_;
 		ValaIntegerType* int_type;
+		ValaFormalParameter* _tmp4_;
 		ValaFormalParameter* _tmp5_;
 		ValaFormalParameter* _tmp6_;
-		ValaFormalParameter* _tmp7_;
-		_tmp0_ = NULL;
-		self->priv->move_method = (_tmp0_ = vala_array_move_method_new (vala_code_node_get_source_reference ((ValaCodeNode*) self)), (self->priv->move_method == NULL) ? NULL : (self->priv->move_method = (vala_code_node_unref (self->priv->move_method), NULL)), _tmp0_);
-		_tmp1_ = NULL;
+		self->priv->move_method = (_tmp0_ = vala_array_move_method_new (vala_code_node_get_source_reference ((ValaCodeNode*) self)), _vala_code_node_unref0 (self->priv->move_method), _tmp0_);
 		vala_method_set_return_type ((ValaMethod*) self->priv->move_method, (ValaDataType*) (_tmp1_ = vala_void_type_new (NULL)));
-		(_tmp1_ == NULL) ? NULL : (_tmp1_ = (vala_code_node_unref (_tmp1_), NULL));
+		_vala_code_node_unref0 (_tmp1_);
 		vala_symbol_set_access ((ValaSymbol*) self->priv->move_method, VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
 		vala_method_set_cname ((ValaMethod*) self->priv->move_method, "_vala_array_move");
-		_tmp2_ = NULL;
-		root_symbol = (_tmp2_ = vala_code_context_get_root (vala_source_file_get_context (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) self)))), (_tmp2_ == NULL) ? NULL : vala_code_node_ref (_tmp2_));
-		_tmp3_ = NULL;
-		_tmp4_ = NULL;
-		int_type = (_tmp4_ = vala_integer_type_new (_tmp3_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) root_symbol), "int")), NULL, NULL), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (vala_code_node_unref (_tmp3_), NULL)), _tmp4_);
-		_tmp5_ = NULL;
-		vala_method_add_parameter ((ValaMethod*) self->priv->move_method, _tmp5_ = vala_formal_parameter_new ("src", (ValaDataType*) int_type, NULL));
-		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (vala_code_node_unref (_tmp5_), NULL));
-		_tmp6_ = NULL;
-		vala_method_add_parameter ((ValaMethod*) self->priv->move_method, _tmp6_ = vala_formal_parameter_new ("dest", (ValaDataType*) int_type, NULL));
-		(_tmp6_ == NULL) ? NULL : (_tmp6_ = (vala_code_node_unref (_tmp6_), NULL));
-		_tmp7_ = NULL;
-		vala_method_add_parameter ((ValaMethod*) self->priv->move_method, _tmp7_ = vala_formal_parameter_new ("length", (ValaDataType*) int_type, NULL));
-		(_tmp7_ == NULL) ? NULL : (_tmp7_ = (vala_code_node_unref (_tmp7_), NULL));
-		(root_symbol == NULL) ? NULL : (root_symbol = (vala_code_node_unref (root_symbol), NULL));
-		(int_type == NULL) ? NULL : (int_type = (vala_code_node_unref (int_type), NULL));
+		root_symbol = _vala_code_node_ref0 (vala_code_context_get_root (vala_source_file_get_context (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) self)))));
+		int_type = (_tmp3_ = vala_integer_type_new (_tmp2_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) root_symbol), "int")), NULL, NULL), _vala_code_node_unref0 (_tmp2_), _tmp3_);
+		vala_method_add_parameter ((ValaMethod*) self->priv->move_method, _tmp4_ = vala_formal_parameter_new ("src", (ValaDataType*) int_type, NULL));
+		_vala_code_node_unref0 (_tmp4_);
+		vala_method_add_parameter ((ValaMethod*) self->priv->move_method, _tmp5_ = vala_formal_parameter_new ("dest", (ValaDataType*) int_type, NULL));
+		_vala_code_node_unref0 (_tmp5_);
+		vala_method_add_parameter ((ValaMethod*) self->priv->move_method, _tmp6_ = vala_formal_parameter_new ("length", (ValaDataType*) int_type, NULL));
+		_vala_code_node_unref0 (_tmp6_);
+		_vala_code_node_unref0 (root_symbol);
+		_vala_code_node_unref0 (int_type);
 	}
-	_tmp8_ = NULL;
-	return (_tmp8_ = self->priv->move_method, (_tmp8_ == NULL) ? NULL : vala_code_node_ref (_tmp8_));
+	result = _vala_code_node_ref0 (self->priv->move_method);
+	return result;
 }
 
 
 static ValaDataType* vala_array_type_real_copy (ValaDataType* base) {
 	ValaArrayType * self;
+	ValaDataType* result;
 	ValaDataType* _tmp0_;
 	ValaArrayType* _tmp1_;
 	ValaArrayType* _result_;
 	self = (ValaArrayType*) base;
-	_tmp0_ = NULL;
-	_tmp1_ = NULL;
-	_result_ = (_tmp1_ = vala_array_type_new (_tmp0_ = vala_data_type_copy (vala_array_type_get_element_type (self)), self->priv->_rank, vala_code_node_get_source_reference ((ValaCodeNode*) self)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (vala_code_node_unref (_tmp0_), NULL)), _tmp1_);
+	_result_ = (_tmp1_ = vala_array_type_new (_tmp0_ = vala_data_type_copy (vala_array_type_get_element_type (self)), self->priv->_rank, vala_code_node_get_source_reference ((ValaCodeNode*) self)), _vala_code_node_unref0 (_tmp0_), _tmp1_);
 	vala_data_type_set_value_owned ((ValaDataType*) _result_, vala_data_type_get_value_owned ((ValaDataType*) self));
 	vala_data_type_set_nullable ((ValaDataType*) _result_, vala_data_type_get_nullable ((ValaDataType*) self));
 	vala_data_type_set_floating_reference ((ValaDataType*) _result_, vala_data_type_get_floating_reference ((ValaDataType*) self));
+	vala_array_type_set_inline_allocated (_result_, self->priv->_inline_allocated);
 	if (self->priv->_fixed_length) {
 		vala_array_type_set_fixed_length (_result_, TRUE);
 		vala_array_type_set_length (_result_, self->priv->_length);
 	}
-	return (ValaDataType*) _result_;
+	result = (ValaDataType*) _result_;
+	return result;
 }
 
 
 static char* vala_array_type_real_get_cname (ValaDataType* base) {
 	ValaArrayType * self;
+	char* result;
 	self = (ValaArrayType*) base;
-	/* FIXME add support for [Immutable] or [Const] attribute to support arrays to const data*/
-	if (self->priv->_fixed_length) {
-		return vala_data_type_get_cname (vala_array_type_get_element_type (self));
+	if (self->priv->_inline_allocated) {
+		result = vala_data_type_get_cname (vala_array_type_get_element_type (self));
+		return result;
 	} else {
+		char* _tmp0_;
 		char* _tmp1_;
-		char* _tmp2_;
-		_tmp1_ = NULL;
-		_tmp2_ = NULL;
-		return (_tmp2_ = g_strconcat (_tmp1_ = vala_data_type_get_cname (vala_array_type_get_element_type (self)), "*", NULL), _tmp1_ = (g_free (_tmp1_), NULL), _tmp2_);
+		result = (_tmp1_ = g_strconcat (_tmp0_ = vala_data_type_get_cname (vala_array_type_get_element_type (self)), "*", NULL), _g_free0 (_tmp0_), _tmp1_);
+		return result;
 	}
 }
 
 
 static char* vala_array_type_real_get_cdeclarator_suffix (ValaDataType* base) {
 	ValaArrayType * self;
+	char* result;
 	self = (ValaArrayType*) base;
 	if (self->priv->_fixed_length) {
-		return g_strdup_printf ("[%d]", self->priv->_length);
+		result = g_strdup_printf ("[%d]", self->priv->_length);
+		return result;
 	} else {
-		return g_strdup ("");
+		if (self->priv->_inline_allocated) {
+			result = g_strdup ("[]");
+			return result;
+		} else {
+			result = g_strdup ("");
+			return result;
+		}
 	}
 }
 
 
 static gboolean vala_array_type_real_is_array (ValaDataType* base) {
 	ValaArrayType * self;
+	gboolean result;
 	self = (ValaArrayType*) base;
-	return TRUE;
+	result = TRUE;
+	return result;
 }
 
 
 static char* vala_array_type_real_to_qualified_string (ValaDataType* base, ValaScope* scope) {
 	ValaArrayType * self;
+	char* result;
 	const char* _tmp0_;
 	char* _tmp2_;
 	char* _tmp1_;
@@ -785,92 +782,109 @@ static char* vala_array_type_real_to_qualified_string (ValaDataType* base, ValaS
 	} else {
 		_tmp0_ = "";
 	}
-	_tmp2_ = NULL;
-	_tmp1_ = NULL;
-	_tmp3_ = NULL;
-	return (_tmp3_ = g_strdup_printf ("%s[%s]%s", _tmp1_ = vala_data_type_to_qualified_string (vala_array_type_get_element_type (self), scope), _tmp2_ = g_strnfill ((gulong) (self->priv->_rank - 1), ','), _tmp0_), _tmp2_ = (g_free (_tmp2_), NULL), _tmp1_ = (g_free (_tmp1_), NULL), _tmp3_);
+	result = (_tmp3_ = g_strdup_printf ("%s[%s]%s", _tmp1_ = vala_data_type_to_qualified_string (vala_array_type_get_element_type (self), scope), _tmp2_ = g_strnfill ((gsize) (self->priv->_rank - 1), ','), _tmp0_), _g_free0 (_tmp2_), _g_free0 (_tmp1_), _tmp3_);
+	return result;
 }
 
 
 static gboolean vala_array_type_real_compatible (ValaDataType* base, ValaDataType* target_type) {
 	ValaArrayType * self;
+	gboolean result;
 	gboolean _tmp0_;
-	ValaArrayType* _tmp6_;
-	ValaDataType* _tmp5_;
+	char* _tmp1_;
+	gboolean _tmp2_;
+	gboolean _tmp5_;
+	ValaDataType* _tmp8_;
 	ValaArrayType* target_array_type;
-	gboolean _tmp8_;
-	gboolean _tmp10_;
+	gboolean _tmp9_;
 	self = (ValaArrayType*) base;
 	g_return_val_if_fail (target_type != NULL, FALSE);
 	_tmp0_ = FALSE;
-	if (VALA_IS_POINTER_TYPE (target_type)) {
-		_tmp0_ = TRUE;
+	if ((_tmp2_ = _vala_strcmp0 (_tmp1_ = vala_data_type_get_type_id (target_type), "G_TYPE_VALUE") == 0, _g_free0 (_tmp1_), _tmp2_)) {
+		ValaSymbol* _tmp4_;
+		ValaCodeContext* _tmp3_;
+		_tmp0_ = VALA_SYMBOL (vala_data_type_get_data_type (vala_array_type_get_element_type (self))) == (_tmp4_ = vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) vala_code_context_get_root (_tmp3_ = vala_code_context_get ())), "string"));
+		_vala_code_node_unref0 (_tmp4_);
+		_vala_code_context_unref0 (_tmp3_);
 	} else {
-		gboolean _tmp1_;
-		_tmp1_ = FALSE;
-		if (vala_data_type_get_data_type (target_type) != NULL) {
-			ValaAttribute* _tmp2_;
-			_tmp2_ = NULL;
-			_tmp1_ = (_tmp2_ = vala_code_node_get_attribute ((ValaCodeNode*) vala_data_type_get_data_type (target_type), "PointerType")) != NULL;
-			(_tmp2_ == NULL) ? NULL : (_tmp2_ = (vala_code_node_unref (_tmp2_), NULL));
-		} else {
-			_tmp1_ = FALSE;
-		}
-		_tmp0_ = _tmp1_;
+		_tmp0_ = FALSE;
 	}
 	if (_tmp0_) {
-		/* any array type can be cast to a generic pointer */
-		return TRUE;
+		result = TRUE;
+		return result;
 	}
-	/* temporarily ignore type parameters */
-	if (vala_data_type_get_type_parameter (target_type) != NULL) {
-		return TRUE;
-	}
-	_tmp6_ = NULL;
-	_tmp5_ = NULL;
-	target_array_type = (_tmp6_ = (_tmp5_ = target_type, VALA_IS_ARRAY_TYPE (_tmp5_) ? ((ValaArrayType*) _tmp5_) : NULL), (_tmp6_ == NULL) ? NULL : vala_code_node_ref (_tmp6_));
-	if (target_array_type == NULL) {
-		gboolean _tmp7_;
-		return (_tmp7_ = FALSE, (target_array_type == NULL) ? NULL : (target_array_type = (vala_code_node_unref (target_array_type), NULL)), _tmp7_);
-	}
-	_tmp8_ = FALSE;
-	if (vala_data_type_compatible (vala_array_type_get_element_type (self), vala_array_type_get_element_type (target_array_type))) {
-		_tmp8_ = vala_data_type_compatible (vala_array_type_get_element_type (target_array_type), vala_array_type_get_element_type (self));
+	_tmp5_ = FALSE;
+	if (VALA_IS_POINTER_TYPE (target_type)) {
+		_tmp5_ = TRUE;
 	} else {
-		_tmp8_ = FALSE;
+		gboolean _tmp6_;
+		_tmp6_ = FALSE;
+		if (vala_data_type_get_data_type (target_type) != NULL) {
+			ValaAttribute* _tmp7_;
+			_tmp6_ = (_tmp7_ = vala_code_node_get_attribute ((ValaCodeNode*) vala_data_type_get_data_type (target_type), "PointerType")) != NULL;
+			_vala_code_node_unref0 (_tmp7_);
+		} else {
+			_tmp6_ = FALSE;
+		}
+		_tmp5_ = _tmp6_;
 	}
-	if (_tmp8_) {
-		gboolean _tmp9_;
-		return (_tmp9_ = TRUE, (target_array_type == NULL) ? NULL : (target_array_type = (vala_code_node_unref (target_array_type), NULL)), _tmp9_);
+	if (_tmp5_) {
+		result = TRUE;
+		return result;
 	}
-	return (_tmp10_ = FALSE, (target_array_type == NULL) ? NULL : (target_array_type = (vala_code_node_unref (target_array_type), NULL)), _tmp10_);
+	if (vala_data_type_get_type_parameter (target_type) != NULL) {
+		result = TRUE;
+		return result;
+	}
+	target_array_type = _vala_code_node_ref0 ((_tmp8_ = target_type, VALA_IS_ARRAY_TYPE (_tmp8_) ? ((ValaArrayType*) _tmp8_) : NULL));
+	if (target_array_type == NULL) {
+		result = FALSE;
+		_vala_code_node_unref0 (target_array_type);
+		return result;
+	}
+	_tmp9_ = FALSE;
+	if (vala_data_type_compatible (vala_array_type_get_element_type (self), vala_array_type_get_element_type (target_array_type))) {
+		_tmp9_ = vala_data_type_compatible (vala_array_type_get_element_type (target_array_type), vala_array_type_get_element_type (self));
+	} else {
+		_tmp9_ = FALSE;
+	}
+	if (_tmp9_) {
+		result = TRUE;
+		_vala_code_node_unref0 (target_array_type);
+		return result;
+	}
+	result = FALSE;
+	_vala_code_node_unref0 (target_array_type);
+	return result;
 }
 
 
 static gboolean vala_array_type_real_is_reference_type_or_type_parameter (ValaDataType* base) {
 	ValaArrayType * self;
+	gboolean result;
 	self = (ValaArrayType*) base;
-	return TRUE;
+	result = TRUE;
+	return result;
 }
 
 
 static char* vala_array_type_real_get_type_signature (ValaDataType* base) {
 	ValaArrayType * self;
+	char* result;
 	char* element_type_signature;
+	char* _tmp0_;
 	char* _tmp1_;
-	char* _tmp2_;
-	char* _tmp3_;
 	self = (ValaArrayType*) base;
 	element_type_signature = vala_data_type_get_type_signature (vala_array_type_get_element_type (self));
 	if (element_type_signature == NULL) {
-		char* _tmp0_;
-		_tmp0_ = NULL;
-		return (_tmp0_ = NULL, element_type_signature = (g_free (element_type_signature), NULL), _tmp0_);
+		result = NULL;
+		_g_free0 (element_type_signature);
+		return result;
 	}
-	_tmp1_ = NULL;
-	_tmp2_ = NULL;
-	_tmp3_ = NULL;
-	return (_tmp3_ = (_tmp2_ = g_strconcat (_tmp1_ = g_strnfill ((gulong) self->priv->_rank, 'a'), element_type_signature, NULL), _tmp1_ = (g_free (_tmp1_), NULL), _tmp2_), element_type_signature = (g_free (element_type_signature), NULL), _tmp3_);
+	result = (_tmp1_ = g_strconcat (_tmp0_ = g_strnfill ((gsize) self->priv->_rank, 'a'), element_type_signature, NULL), _g_free0 (_tmp0_), _tmp1_);
+	_g_free0 (element_type_signature);
+	return result;
+	_g_free0 (element_type_signature);
 }
 
 
@@ -895,31 +909,95 @@ static void vala_array_type_real_replace_type (ValaCodeNode* base, ValaDataType*
 
 static GeeList* vala_array_type_real_get_symbols (ValaDataType* base) {
 	ValaArrayType * self;
+	GeeList* result;
 	self = (ValaArrayType*) base;
-	return vala_data_type_get_symbols (vala_array_type_get_element_type (self));
+	result = vala_data_type_get_symbols (vala_array_type_get_element_type (self));
+	return result;
+}
+
+
+static gboolean vala_array_type_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
+	ValaArrayType * self;
+	gboolean result;
+	self = (ValaArrayType*) base;
+	g_return_val_if_fail (analyzer != NULL, FALSE);
+	if (self->priv->_invalid_syntax) {
+		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), "syntax error, no expression allowed between array brackets");
+		vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
+		result = FALSE;
+		return result;
+	}
+	result = vala_code_node_check ((ValaCodeNode*) vala_array_type_get_element_type (self), analyzer);
+	return result;
+}
+
+
+static char* vala_array_type_real_get_type_id (ValaDataType* base) {
+	ValaArrayType * self;
+	char* result;
+	ValaSymbol* _tmp1_;
+	ValaCodeContext* _tmp0_;
+	gboolean _tmp2_;
+	self = (ValaArrayType*) base;
+	if ((_tmp2_ = VALA_SYMBOL (vala_data_type_get_data_type (vala_array_type_get_element_type (self))) == (_tmp1_ = vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) vala_code_context_get_root (_tmp0_ = vala_code_context_get ())), "string")), _vala_code_node_unref0 (_tmp1_), _vala_code_context_unref0 (_tmp0_), _tmp2_)) {
+		result = g_strdup ("G_TYPE_STRV");
+		return result;
+	} else {
+		result = NULL;
+		return result;
+	}
 }
 
 
 ValaDataType* vala_array_type_get_element_type (ValaArrayType* self) {
+	ValaDataType* result;
 	g_return_val_if_fail (self != NULL, NULL);
-	return self->priv->_element_type;
+	result = self->priv->_element_type;
+	return result;
 }
 
 
 void vala_array_type_set_element_type (ValaArrayType* self, ValaDataType* value) {
-	ValaDataType* _tmp2_;
-	ValaDataType* _tmp1_;
+	ValaDataType* _tmp0_;
 	g_return_if_fail (self != NULL);
-	_tmp2_ = NULL;
-	_tmp1_ = NULL;
-	self->priv->_element_type = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : vala_code_node_ref (_tmp1_)), (self->priv->_element_type == NULL) ? NULL : (self->priv->_element_type = (vala_code_node_unref (self->priv->_element_type), NULL)), _tmp2_);
+	self->priv->_element_type = (_tmp0_ = _vala_code_node_ref0 (value), _vala_code_node_unref0 (self->priv->_element_type), _tmp0_);
 	vala_code_node_set_parent_node ((ValaCodeNode*) self->priv->_element_type, (ValaCodeNode*) self);
 }
 
 
-gboolean vala_array_type_get_fixed_length (ValaArrayType* self) {
+gboolean vala_array_type_get_invalid_syntax (ValaArrayType* self) {
+	gboolean result;
 	g_return_val_if_fail (self != NULL, FALSE);
-	return self->priv->_fixed_length;
+	result = self->priv->_invalid_syntax;
+	return result;
+}
+
+
+void vala_array_type_set_invalid_syntax (ValaArrayType* self, gboolean value) {
+	g_return_if_fail (self != NULL);
+	self->priv->_invalid_syntax = value;
+}
+
+
+gboolean vala_array_type_get_inline_allocated (ValaArrayType* self) {
+	gboolean result;
+	g_return_val_if_fail (self != NULL, FALSE);
+	result = self->priv->_inline_allocated;
+	return result;
+}
+
+
+void vala_array_type_set_inline_allocated (ValaArrayType* self, gboolean value) {
+	g_return_if_fail (self != NULL);
+	self->priv->_inline_allocated = value;
+}
+
+
+gboolean vala_array_type_get_fixed_length (ValaArrayType* self) {
+	gboolean result;
+	g_return_val_if_fail (self != NULL, FALSE);
+	result = self->priv->_fixed_length;
+	return result;
 }
 
 
@@ -930,8 +1008,10 @@ void vala_array_type_set_fixed_length (ValaArrayType* self, gboolean value) {
 
 
 gint vala_array_type_get_length (ValaArrayType* self) {
+	gint result;
 	g_return_val_if_fail (self != NULL, 0);
-	return self->priv->_length;
+	result = self->priv->_length;
+	return result;
 }
 
 
@@ -942,8 +1022,10 @@ void vala_array_type_set_length (ValaArrayType* self, gint value) {
 
 
 gint vala_array_type_get_rank (ValaArrayType* self) {
+	gint result;
 	g_return_val_if_fail (self != NULL, 0);
-	return self->priv->_rank;
+	result = self->priv->_rank;
+	return result;
 }
 
 
@@ -969,6 +1051,8 @@ static void vala_array_type_class_init (ValaArrayTypeClass * klass) {
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_array_type_real_accept_children;
 	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_array_type_real_replace_type;
 	VALA_DATA_TYPE_CLASS (klass)->get_symbols = vala_array_type_real_get_symbols;
+	VALA_CODE_NODE_CLASS (klass)->check = vala_array_type_real_check;
+	VALA_DATA_TYPE_CLASS (klass)->get_type_id = vala_array_type_real_get_type_id;
 }
 
 
@@ -980,10 +1064,10 @@ static void vala_array_type_instance_init (ValaArrayType * self) {
 static void vala_array_type_finalize (ValaCodeNode* obj) {
 	ValaArrayType * self;
 	self = VALA_ARRAY_TYPE (obj);
-	(self->priv->_element_type == NULL) ? NULL : (self->priv->_element_type = (vala_code_node_unref (self->priv->_element_type), NULL));
-	(self->priv->length_field == NULL) ? NULL : (self->priv->length_field = (vala_code_node_unref (self->priv->length_field), NULL));
-	(self->priv->resize_method == NULL) ? NULL : (self->priv->resize_method = (vala_code_node_unref (self->priv->resize_method), NULL));
-	(self->priv->move_method == NULL) ? NULL : (self->priv->move_method = (vala_code_node_unref (self->priv->move_method), NULL));
+	_vala_code_node_unref0 (self->priv->_element_type);
+	_vala_code_node_unref0 (self->priv->length_field);
+	_vala_code_node_unref0 (self->priv->resize_method);
+	_vala_code_node_unref0 (self->priv->move_method);
 	VALA_CODE_NODE_CLASS (vala_array_type_parent_class)->finalize (obj);
 }
 
